//! @brief calculate on-the-fly statistics
//! @copyright (c) since 2024, see LICENSE.txt
#ifndef OTF_STATS_H
#define OTF_STATS_H

#include <cstdint>
#include <string>

// --------------------
//! holds various stats calculated on the fly for the given set of values
class OTFStats {
public:
    //! constructor
    //! \param[in] tag  used for reporting to indicate what this data is for
    explicit OTFStats(std::string tag);

    //! reset all values to initial state
    void init();

    //! update stats with the given the given value
    //! \param[in]  val   the value to update stats with
    void update(double val);

    //! signature of callback function to print a given line
    using writeln_func = void (&)(const std::string&);

    //! report all stats to stdout
    //! \param[in] writeln      callback function to print the given line
    //! \param[in] do_header    (optional) if true print a header for the report
    void report(writeln_func writeln, bool do_header = false) const;

    //! return tag reporting for this data
    //! \return std::string
    [[nodiscard]] auto tag() const -> std::string;

    //! number of values given; 0 if none
    //! \return 64 bit int
    [[nodiscard]] auto num_elements() const -> int_fast64_t;

    //! average of all values given; NAN if none
    //! \return double
    [[nodiscard]] auto average() const -> double;

    //! min value given; NAN if none
    //! \return double
    [[nodiscard]] auto minimum() const -> double;

    //! max value given; NAN if none
    //! \return double
    [[nodiscard]] auto maximum() const -> double;

    //! standard deviation of sequence given; NAN if fewer than 2 values
    //! \return double
    [[nodiscard]] auto stddev() const -> double;

private:
    //! tag to identify during logging and reporting
    std::string m_tag;

    //! number of values given; 0 if none
    int_fast64_t m_num_elems { 0 };

    // === for min-max
    //! min value given; NAN if none
    double m_min {};
    //! max value given; NAN if none
    double m_max {};

    // === for stddev
    //! current mean (average) of values given; NAN if none
    double m_mean {};
    //! current second mean of values given; NAN if none
    double m_mean2 {};
    //! current variance of values given; NAN if none
    double m_variance {};
    //! current standard deviation of values given; NAN if 0 or 1 values so far
    double m_stddev = {};
};

#endif // OTF_STATS_H
